package com.jiuqi.util.crypto;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.jiuqi.util.JqLib;
import com.jiuqi.util.crypto.EncryptException;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.Key;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.Provider;
import java.security.SecureRandom;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import javax.crypto.Cipher;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

public class RSAUtil {
  public static KeyPair generateKeyPair() throws EncryptException {
    try {
      KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA", 
          (Provider)new BouncyCastleProvider());
      keyPairGen.initialize(1024, new SecureRandom());
      KeyPair keyPair = keyPairGen.genKeyPair();
      return keyPair;
    } catch (Exception e) {
      throw new EncryptException(e.getMessage());
    } 
  }
  
  public static RSAPublicKey generateRSAPublicKey(byte[] modulus, byte[] publicExponent) throws EncryptException {
    KeyFactory keyFac = null;
    try {
      keyFac = KeyFactory.getInstance("RSA", (Provider)new BouncyCastleProvider());
    } catch (NoSuchAlgorithmException ex) {
      throw new EncryptException(ex.getMessage());
    } 
    RSAPublicKeySpec pubKeySpec = new RSAPublicKeySpec(new BigInteger(modulus), new BigInteger(publicExponent));
    try {
      return (RSAPublicKey)keyFac.generatePublic(pubKeySpec);
    } catch (InvalidKeySpecException ex) {
      throw new EncryptException(ex.getMessage());
    } 
  }
  
  public static RSAPrivateKey generateRSAPrivateKey(byte[] modulus, byte[] privateExponent) throws EncryptException {
    KeyFactory keyFac = null;
    try {
      keyFac = KeyFactory.getInstance("RSA", (Provider)new BouncyCastleProvider());
    } catch (NoSuchAlgorithmException ex) {
      throw new EncryptException(ex.getMessage());
    } 
    RSAPrivateKeySpec priKeySpec = new RSAPrivateKeySpec(new BigInteger(modulus), new BigInteger(privateExponent));
    try {
      return (RSAPrivateKey)keyFac.generatePrivate(priKeySpec);
    } catch (InvalidKeySpecException ex) {
      throw new EncryptException(ex.getMessage());
    } 
  }
  
  public static byte[] encrypt(Key key, byte[] data) throws EncryptException {
    try {
      Cipher cipher = Cipher.getInstance("RSA", (Provider)new BouncyCastleProvider());
      cipher.init(1, key);
      int blockSize = cipher.getBlockSize();
      int outputSize = cipher.getOutputSize(data.length);
      int leavedSize = data.length % blockSize;
      int blocksSize = (leavedSize != 0) ? (data.length / blockSize + 1) : (data.length / blockSize);
      byte[] raw = new byte[outputSize * blocksSize];
      int i = 0;
      while (data.length - i * blockSize > 0) {
        if (data.length - i * blockSize > blockSize) {
          cipher.doFinal(data, i * blockSize, blockSize, raw, i * outputSize);
        } else {
          cipher.doFinal(data, i * blockSize, data.length - i * blockSize, raw, i * outputSize);
        } 
        i++;
      } 
      return raw;
    } catch (Exception e) {
      throw new EncryptException(e.getMessage());
    } 
  }
  
  public static byte[] decrypt(Key key, byte[] raw) throws EncryptException {
    try {
      Cipher cipher = Cipher.getInstance("RSA", (Provider)new BouncyCastleProvider());
      cipher.init(2, key);
      int blockSize = cipher.getBlockSize();
      ByteArrayOutputStream bout = new ByteArrayOutputStream(64);
      int j = 0;
      while (raw.length - j * blockSize > 0) {
        bout.write(cipher.doFinal(raw, j * blockSize, blockSize));
        j++;
      } 
      return bout.toByteArray();
    } catch (Exception e) {
      throw new EncryptException(e.getMessage());
    } 
  }
  
  public static void main(String[] args) throws Exception {
    File file = new File("d:\\test.html");
    FileInputStream in = new FileInputStream(file);
    ByteArrayOutputStream bout = new ByteArrayOutputStream();
    byte[] tmpbuf = new byte[1024];
    int count = 0;
    while ((count = in.read(tmpbuf)) != -1) {
      bout.write(tmpbuf, 0, count);
      tmpbuf = new byte[1024];
    } 
    in.close();
    byte[] orgData = bout.toByteArray();
    KeyPair keyPair = generateKeyPair();
    RSAPublicKey pubKey = (RSAPublicKey)keyPair.getPublic();
    RSAPrivateKey priKey = (RSAPrivateKey)keyPair.getPrivate();
    byte[] pubModBytes = pubKey.getModulus().toByteArray();
    byte[] pubPubExpBytes = pubKey.getPublicExponent().toByteArray();
    byte[] priModBytes = priKey.getModulus().toByteArray();
    byte[] priPriExpBytes = priKey.getPrivateExponent().toByteArray();
    RSAPublicKey recoveryPubKey = generateRSAPublicKey(pubModBytes, pubPubExpBytes);
    RSAPrivateKey recoveryPriKey = generateRSAPrivateKey(priModBytes, priPriExpBytes);
    System.out.println(JqLib.bytesToHexString(priModBytes).toString());
    System.out.println(JqLib.bytesToHexString(priPriExpBytes).toString());
    System.out.println(JqLib.bytesToHexString(pubModBytes).toString());
    System.out.println(JqLib.bytesToHexString(pubPubExpBytes).toString());
    byte[] raw = encrypt(recoveryPriKey, orgData);
    file = new File("d:\\encrypt_result.dat");
    OutputStream out = new FileOutputStream(file);
    out.write(raw);
    out.close();
    byte[] data = decrypt(recoveryPubKey, raw);
    file = new File("d:\\decrypt_result.html");
    out = new FileOutputStream(file);
    out.write(data);
    out.flush();
    out.close();
  }
}
